'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.ElementBuilder = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

exports.element = element;

var _types = require('../helper/types.js');

var _functions = require('../helper/functions.js');

var _converters = require('../helper/converters.js');

var _arrays = require('../helper/arrays.js');

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var LIFECYCLE_CALLBACKS = ['createdCallback', 'attachedCallback', 'detachedCallback', 'attributeChangedCallback'];

var LIFECYCLE_EVENTS = (0, _arrays.flatten)(LIFECYCLE_CALLBACKS.map(function (name) {
    return ['before:' + name, 'after:' + name];
}));

function applyLifecycle(context, name) {
    var proto = context.p,
        original = proto[name],
        beforeFns = context.events['before:' + name],
        afterFns = context.events['after:' + name];

    proto[name] = function () {
        var _this = this;

        var args = [this].concat((0, _converters.toArray)(arguments));

        beforeFns.forEach(function (fn) {
            return fn.apply(_this, args);
        });

        if ((0, _types.isFunction)(original)) {
            original.apply(this, args);
        }

        afterFns.forEach(function (fn) {
            return fn.apply(_this, args);
        });
    };
}

/**
 * The custom element builder.
 * Its goal is to provide a user friendly way to do it by some else (i.e. dedicated builders).
 */

var ElementBuilder = function () {

    /**
     */

    function ElementBuilder() {
        _classCallCheck(this, ElementBuilder);

        var p = Object.create(HTMLElement.prototype),
            builders = [],
            events = LIFECYCLE_EVENTS.reduce(function (a, b) {
            a[b] = [];
            return a;
        }, {
            'before:builders': [],
            'after:builders': [],
            'before:registerElement': [],
            'after:registerElement': []
        });
        /**
         * @type {Object}
         * @property {!Object} p - the prototype
         * @property {!string} e - the name of a native element
         * @desc the context of the builder
         */
        this.context = { p: p, builders: builders, events: events };
    }

    /**
     * Set the basement of the future custom element, i.e. the prototype and/or the extends value.
     * Prototype and extends value can be swapped.
     * @example
     * element().base(prototypeValue, extendsValue);
     * element().base(extendsValue, prototypeValue);
     * element().base(extendsValue);
     * element().base(prototypeValue);
     * @param {!(string|Object)} arg1 the prototype or the name of the native element
     * @param {string|Object} [arg2] the prototype or the name of the native element
     * @returns {ElementBuilder} the builder
     */


    _createClass(ElementBuilder, [{
        key: 'base',
        value: function base(arg1, arg2) {
            var arg1Type = typeof arg1 === 'undefined' ? 'undefined' : _typeof(arg1);
            var p = arg1Type === 'string' ? arg2 : arg1;
            var e = arg1Type === 'string' ? arg1 : arg2;
            if (p) {
                this.context.p = p;
            }
            if (e) {
                this.context.e = e;
            }
            return this;
        }

        /**
         * To apply the given builders during the build process.
         * @param {...Builder} builders the builders
         * @returns {ElementBuilder} the builder
         */

    }, {
        key: 'builders',
        value: function builders() {
            var _this2 = this;

            for (var _len = arguments.length, _builders = Array(_len), _key = 0; _key < _len; _key++) {
                _builders[_key] = arguments[_key];
            }

            _builders.forEach(function (builder) {
                return _this2.context.builders.push(builder);
            });
            return this;
        }

        /**
         * To register call back on events.
         * @param {!string} event the event name
         * @returns {Object} the on builder.
         * @property {function(callback: function)} invoke - to register the callback
         */

    }, {
        key: 'on',
        value: function on(event) {
            var _this3 = this;

            var invoke = function invoke(cb) {
                _this3.context.events[event].push(cb);
                return _this3;
            };
            return { invoke: invoke };
        }

        /**
         * To register the custom element.
         * @param {!string} name the name of the cutsom element
         * @returns {Element} the custom element Type
         */

    }, {
        key: 'register',
        value: function register(name) {
            var _this4 = this;

            this.context.events['before:builders'].forEach(function (fn) {
                return fn(_this4.context);
            });

            (0, _arrays.invoke)(this.context.builders, 'build', this.context.p, (0, _functions.bind)(this.on, this));

            this.context.events['after:builders'].forEach(function (fn) {
                return fn(_this4.context);
            });

            LIFECYCLE_CALLBACKS.forEach((0, _functions.partial)(applyLifecycle, this.context));

            var options = { prototype: this.context.p };

            if ((0, _types.isString)(this.context.e)) {
                options.extends = this.context.e;
            }

            this.context.events['before:registerElement'].forEach(function (fn) {
                return fn(_this4.context);
            });

            var CustomElement = document.registerElement(name, options);

            this.context.events['after:registerElement'].forEach(function (fn) {
                return fn(CustomElement);
            });

            return CustomElement;
        }
    }]);

    return ElementBuilder;
}();

/**
 * Get a new custom element builder.
 * @returns {ElementBuilder} the custom element builder
 */


exports.ElementBuilder = ElementBuilder;
function element() {
    return new ElementBuilder();
}