'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.PropertyBuilder = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

exports.property = property;

var _types = require('../helper/types.js');

var _objects = require('../helper/objects.js');

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var DEFAULT_DATA = {
    enumerable: true,
    immutable: false,
    descriptorValue: true
};

/**
 * The property builder.
 * Its goal is to provide a way to define a property.
 */

var PropertyBuilder = function () {

    /**
     * @param {!string} propName the name of the property
     */

    function PropertyBuilder(propName) {
        _classCallCheck(this, PropertyBuilder);

        /**
         * @ignore
         */
        this.data = (0, _objects.assign)({ propName: propName, listeners: [] }, DEFAULT_DATA);
    }

    /**
     * To make an immutable property.
     * @returns {PropertyBuilder} the builder
     */


    _createClass(PropertyBuilder, [{
        key: 'immutable',
        value: function immutable() {
            this.data.immutable = true;
            return this;
        }

        /**
         * To hide the property name when using <code>Object.keys()</code>.
         * @returns {PropertyBuilder} the builder
         */

    }, {
        key: 'hidden',
        value: function hidden() {
            this.data.enumerable = false;
            return this;
        }

        /**
         * To set a default value.
         * @param {*} value the default value
         * @returns {PropertyBuilder} the builder
         */

    }, {
        key: 'value',
        value: function value(_value) {
            this.data.value = _value;
            return this;
        }

        /**
         * To set a getter function.
         * @param {function(el: HTMLElement): *} fn the getter function
         * @returns {PropertyBuilder} the builder
         */

    }, {
        key: 'getter',
        value: function getter(fn) {
            this.data.descriptorValue = false;
            this.data.getter = fn;
            return this;
        }

        /**
         * To set a setter function.
         * @param {function(el: HTMLElement, value: *)} fn the setter function
         * @returns {PropertyBuilder} the builder
         */

    }, {
        key: 'setter',
        value: function setter(fn) {
            this.data.descriptorValue = false;
            this.data.setter = fn;
            return this;
        }

        /**
         * To be notified when the property is updated.
         * @param {function(el: HTMLElement, oldVal: *, newVal: *)} listener the listener function
         * @returns {PropertyBuilder} the builder
         */

    }, {
        key: 'listen',
        value: function listen(listener) {
            this.data.listeners.push(listener);
            return this;
        }

        /**
         * Logic of the builder.
         * @param {!ElementBuilder.context.proto} proto the prototype
         * @param {!ElementBuilder.on} on the method on
         */

    }, {
        key: 'build',
        value: function build(proto, on) {
            var data = this.data,
                defaultValue = (0, _objects.result)(this.data, 'value'),
                descriptor = {
                enumerable: this.data.enumerable
            };

            if (this.data.immutable) {
                descriptor.configurable = false;
                descriptor.writable = false;
            } else if ((0, _types.isFunction)(this.data.getter) || (0, _types.isFunction)(this.data.setter)) {
                descriptor.configurable = false;
                descriptor.get = function () {
                    if (data.getter) {
                        return data.getter.call(this, this);
                    }
                };
                descriptor.set = function (value) {
                    if (data.setter) {
                        return data.setter.call(this, this, value);
                    }
                };
            } else {
                descriptor.configurable = true;
                descriptor.writable = true;
            }

            if (data.listeners.length > 0) {
                (function () {
                    descriptor.configurable = false;
                    delete descriptor.writable;
                    data.descriptorValue = false;
                    var _propName = '__' + data.propName + 'LastSetValue';
                    if (!descriptor.get) {
                        descriptor.get = function () {
                            return this[_propName];
                        };
                    }
                    descriptor.set = function (newVal) {
                        var _this = this;

                        var oldVal = this[_propName];
                        this[_propName] = newVal;
                        if (data.setter) {
                            data.setter.call(this, this, newVal);
                        }
                        data.listeners.forEach(function (listener) {
                            listener.call(_this, _this, oldVal, newVal);
                        });
                    };
                })();
            }

            if (data.descriptorValue) {
                descriptor.value = defaultValue;
            }

            Object.defineProperty(proto, this.data.propName, descriptor);

            on('after:createdCallback').invoke(function (el) {
                if (!data.descriptorValue && !(0, _types.isUndefined)(defaultValue)) {
                    el[data.propName] = defaultValue;
                }
            });
        }
    }]);

    return PropertyBuilder;
}();

/**
 * Get a new property builder.
 * @param {!string} propName the name of the property
 * @returns {PropertyBuilder} the property builder
 */


exports.PropertyBuilder = PropertyBuilder;
function property(propName) {
    return new PropertyBuilder(propName);
}