'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.flatten = flatten;
exports.invoke = invoke;

var _types = require('./types.js');

/**
 * Flattens a nested array.
 * @param {!Array} array the array to flatten
 * @returns {Array} the new flattened array
 */
function flatten(array) {
    return array.reduce(function (a, b) {
        return (0, _types.isArray)(b) ? a.concat(flatten(b)) : a.concat(b);
    }, []);
}

/**
 * For each objects, invoke the method called methName with the arguments args.
 * @param {!Array<Object>} objects the objects
 * @param {!string} methName the name of the method
 * @param {...*} args the arguments to invoke the method with
 * @return {Array} the array of results
 */
// obviously close to underscorejs and lodash ;)

function invoke(objects, methName) {
    for (var _len = arguments.length, args = Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
        args[_key - 2] = arguments[_key];
    }

    if ((0, _types.isArray)(objects)) {
        return objects.filter(function (obj) {
            return (0, _types.isFunction)(obj[methName]);
        }).map(function (obj) {
            return obj[methName].apply(obj, args);
        });
    }
    return [];
}