'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.partial = partial;
exports.bind = bind;
exports.noop = noop;

var _converters = require('./converters.js');

/**
 * Partially apply a function by filling in any number of its arguments, without changing its dynamic this value.
 * @param {!Function} fn the function to partially apply arguments to
 * @param {...} args the arguments to be partially applied
 * @returns {Function} the new partially applied function
 */
function partial(fn) {
    for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
        args[_key - 1] = arguments[_key];
    }

    return function () {
        return fn.apply(this, args.concat((0, _converters.toArray)(arguments)));
    };
}

/**
 * Creates a function that invokes fn with context the context ctx.
 * @param {!Function} fn the function to bind
 * @param ctx the context
 * @returns {Function} the new bound function
 */
// obviously close to underscorejs and lodash ;)

function bind(fn, ctx) {
    return function () {
        return fn.apply(ctx, (0, _converters.toArray)(arguments));
    };
}

/**
 * An empty function doing nothing.
 * @returns {Function} a new function doing nothing
 */
function noop() {
    return function () {};
}